// Firefox Extension Conflict Troubleshooter License (FECT)
// author: Chuck Baker
// contact: firefox@customsoftwareconsult.com
// Version 1.0

var EXPORTED_SYMBOLS = [ "FECT" ];

if (typeof(FECT) == "undefined") var FECT = {};

// Global variables
var Cc = Components.classes;
var Ci = Components.interfaces;
var Cu = Components.utils;

Cu.import("resource://gre/modules/AddonManager.jsm");
Cu.import("resource://gre/modules/FileUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");

window.addEventListener('load',function(){FECT.init()},true);

// Replacement for deprecated Application.storage
var FECTstorage = {
	topWin: Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser"),  
	setItem: function setItem(key, value){
			if(typeof this.topWin.storage == "undefined") this.topWin.storage = {};
			if(key == null || key == "") return false;
			this.topWin.storage[key] = value;
			return true;
	},
	getItem: function getItem(key,dFault){
			if(key == null || key == "") return false;
			if(typeof this.topWin.storage[key] == "undefined") return dFault;
			let value = this.topWin.storage[key];
			return value;
	}
}

FECT = {
debug: false,
allAddons: [],
nameGuidXref: {},
fectClipboard: [],
FECT_GUID: "fect@softwarebychuck",
fectMsg: [],						// Array holding localized strings
fectNamespace: 'http://www.w3.org/1999/xhtml',	// HTML5 namespace
fectHideHelp:	false,
fectPlaySounds: false,
fectPlaySoundsURI: {},	// Location of sounds
DBname: "FECT.sqlite",
DBfile: FileUtils.getFile("ProfD", ["FECT.sqlite"]),
DB: {},
sessionID: "",
iteration: 0,
fectMainWindow: {},
fectSessionWindow: {},
fectWin: {},
targetExt: "",
fectProfDir: {},
fectInstallDirectory: "",
extensionsArray: {},
contEnabledList: [],
contDisabledList: [],
contSuspectList: [],
resloved: false,
mainWin: Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator).getMostRecentWindow("navigator:browser"), 


// Get pointer to preferences
fectPrefs: Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefService).getBranch(""),

fectGetUnicharPref: function (prefName) {
	let tmp = FECT.fectPrefs.getComplexValue(prefName, Ci.nsISupportsString).data;
	return tmp;
},

fectSetUnicharPref: function (prefName, prefValue) {
	let supportsString = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
	supportsString.data = prefValue;
	FECT.fectPrefs.setComplexValue(prefName, Ci.nsISupportsString, supportsString);
	return true;
},

fectInitMsgs: function (){
  // Localized error messages (See chrome://pasteemail/locale/pasteemail.properties)
  // Error messages, prompts, etc.
	
	if(!FECT.fectMsg[0]){
		let bundle = Services.strings.createBundle("chrome://fect/locale/FECT.properties");
		let enumerator = bundle.getSimpleEnumeration();
		while (enumerator.hasMoreElements()) {
			let string = enumerator.getNext().QueryInterface(Ci.nsIPropertyElement);
			let stringName = string.key;
			if(stringName.substr(0,3) != "msg") continue;
			let index = stringName.substr(3);
			index = parseInt(index);
			let value = string.value;
			FECT.fectMsg[index] = value;
		}
	}	
	return true;
},

init: function(){
	FECT.fectGetPrefs();
	FECT.fectInitMsgs();
	
	let windowtype = document.documentElement.getAttribute("windowtype");
	if(windowtype != "navigator:browser") return true;
	FECT.fectGetAddonList();

	if(!FECT.DBfile.exists()) FECT.createDB();
	
	// Get FECT version
	AddonManager.getAddonByID(FECT.FECT_GUID, function(addon){
		FECT.fectPrefs.setCharPref("extensions.fect.version",addon.version);
	});
	let currentSessionID = FECT.fectPrefs.getCharPref("extensions.fect.currentSessionID");
	let sessionInProgress = (currentSessionID != "") ? true : false;
	FECT.setMenu(sessionInProgress);
	
	window.removeEventListener("load",FECT.init,true);
	return true;
},

setMenu: function(sessionInProgress){
	FECT.mainWin.document.getElementById("fect_tools_menu_start_session").disabled = sessionInProgress;
	FECT.mainWin.document.getElementById("fect_tools_menu_continue_session").disabled = !sessionInProgress;
	FECT.mainWin.document.getElementById("fect_tools_menu_end_session").disabled = !sessionInProgress;

	try{
		FECT.mainWin.document.getElementById("fect_start_session").disabled = sessionInProgress;
		FECT.mainWin.document.getElementById("fect_continue_session").disabled = !sessionInProgress;
		FECT.mainWin.document.getElementById("fect_end_session").disabled = !sessionInProgress;
	}catch(err){;;}
	return true;
},

fectGetPrefs: function (){
  // Pointer to main browser window
	let wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);  
	FECT.fectMainWindow = wm.getMostRecentWindow("navigator:browser"); 

  // Get/set the preferences
	FECT.fectHideHelp = FECT.fectPrefs.getBoolPref("extensions.fect.hidehelp");
	//FECT.fectHideHelpIcons();
	FECT.fectPlaySounds = FECT.fectPrefs.getBoolPref("extensions.fect.playSounds");
	
	// Create pointer to profile directory
	FECT.fectProfDir = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("ProfD", Ci.nsIFile);
	
	// Find FECT install location
	// First look in profile/extensions
	FECT.fectInstallDirectory = null;
	let dir = FECT.fectProfDir.clone();
	dir.append("extensions");
	dir.append(FECT.FECT_GUID);
	if(dir.exists()){
		FECT.fectInstallDirectory = dir;
	}else{
		// Not there, look in Fx install/browser/extensions
		dir = Cc["@mozilla.org/file/directory_service;1"].getService(Ci.nsIProperties).get("CurProcD", Ci.nsIFile);
		dir.append("extensions");
		dir.append(FECT.FECT_GUID);
		if(dir.exists()){
			FECT.fectInstallDirectory = dir;
		}
	}
	if(!FECT.fectInstallDirectory) alert("Cannot find FECT install location")

	return true;
},

getCurrentSessionInfo: function(){
	FECT.sessionID = FECT.fectPrefs.getCharPref("extensions.fect.currentSessionID");
	FECT.iteration = FECT.fectPrefs.getIntPref("extensions.fect.currentIteneration");
	FECT.targetExt = FECT.fectGetUnicharPref("extensions.fect.currentTargetExt");
	return true;
},

fectResetPrefs: function(){
	FECT.fectPrefs.setCharPref("extensions.fect.currentSessionID", "");	
	FECT.fectPrefs.setIntPref("extensions.fect.currentIteneration", 0);	
	FECT.fectPrefs.setCharPref("extensions.fect.currentTargetExt", "");	
},

fectSetPrefs: function(){
	FECT.fectPrefs.setCharPref("extensions.fect.currentSessionID", FECT.sessionID);
	FECT.fectSetUnicharPref("extensions.fect.currentTargetExt", FECT.targetExt);
	FECT.fectPrefs.setIntPref("extensions.fect.currentIteneration", FECT.iteration);
},

fectGetAddonList: function(){
	FECT.allAddons = [];
	FECT.nameGuidXref = {};
	let nameArray = [];
	let guidArray = {};
	let delimiter = '\u2763';
	let cnt = 0;
	let dcnt = 0;
	let ecnt = 0;
	AddonManager.getAllAddons(function(addons){
		addons.forEach(function(addon){
			let item = {};
			for(let property in addon){	// Filter out unreachable properties
				try{
					item[property] = addon[property];
				}catch(err){
					item[property] = null;
				}
			}
			
			// Exclude certain extensions when in debug mode
			var exclude = "{fe9e8cf0-9737-11dd-bb40-0011098efe97} ";// Bottom error console
			exclude += "{75739dec-72db-4020-aa9a-6afa6744759b} "; 	// Developer assistant
			exclude += "{1280606b-2510-4fe0-97ef-9b5a22eafe80} "; 	// Console
			exclude += "inspector@mozilla.org "; 										//  DOM Inspector 
			exclude += "restartbutton@strk.jp "; 										//  Restart button
			exclude += "SQLiteManager@mrinalkant.blogspot.com "; 		// SQLite Manager
			exclude += "status4evar@caligonstudios.com "; 					// Status-4-Evar 
			exclude += "{dc572301-7619-498c-a57d-39143191b318} "; 	// Tab Mix Plus
			if(!FECT.debug) exclude = "";
			
			if(item.type == 'extension' && exclude.indexOf(item.id) == -1){
				if(addon.id != FECT.FECT_GUID){
					nameArray.push(item.name+delimiter+item.id);
					guidArray[item.id] = item;
					FECT.nameGuidXref[item.name] = item.id;
					cnt++;
					if(item.userDisabled) dcnt++;
					if(!item.userDisabled) ecnt++;
				}
			}
		});

		function caseInsensitive(a, b) {
			return a.toLowerCase().localeCompare(b.toLowerCase());  
		};

		nameArray.sort(caseInsensitive);
		for(let i in nameArray){
			let key = nameArray[i];
			let pos = key.indexOf(delimiter)+1
			let guid = key.substr(pos);
			let item = guidArray[guid];
			FECT.allAddons.push(item);
		}
		FECTstorage.setItem("fectAllAddons", FECT.allAddons);
		FECTstorage.setItem("fectNameGuidXref", FECT.nameGuidXref);
	});
	return true;
},

buildTargetList: function(){	
	// Populate the session target dropdown list
	FECT.fectInitMsgs();
	FECT.allAddons = FECTstorage.getItem("fectAllAddons", FECT.allAddons);
	FECT.clearListBox('fectTargetList');
	let list = document.getElementById('fectTargetList');
	let cnt = 0;
	let targetPool = [];
	for(let i in FECT.allAddons){
		let item = FECT.allAddons[i];
		if(item.id == FECT.FECT_GUID) continue;
		if(item.appDisabled) continue;
		if(item.userDisabled) continue;
		
		let targetExtObj = {};
		targetExtObj.name = item.name;
		targetExtObj.id = item.id;
		targetPool.push(targetExtObj)
		cnt++;
	}
	if(cnt == 0){
		FECT.fectAlert(FECT.fectMsg[0])
		return false;
	}
	for(let i in targetPool){
		let item = targetPool[i];
		let menuitem=document.createElement('menuitem');
		menuitem.setAttribute( "label" , item.name);
		menuitem.setAttribute( "value" , item.id);
		menuitem.onclick = function(event){FECT.dispTargetList(item);};
		list.appendChild(menuitem);
	}
	return true;
},

dispTargetList: function(target){
	// Assign a session Id
	let d = new Date();
	let sessionID = d.getTime();   
	document.getElementById('FECTsessionId').value = sessionID;

	document.getElementById('fectTargetExtName').value = target.name;
	document.getElementById('fectTargetExtID').value = target.id;
	
	let enabledExt = document.getElementById('fectStartEnabledExtList');
	let disabledExt = document.getElementById('fectStartDisabledExtList');
	
	// Clear the lists
	FECT.clearListBox('fectStartEnabledExtList');
	FECT.clearListBox('fectStartDisabledExtList');
	let disCnt = enCnt = 0;
	
	for(let i in FECT.allAddons){
		let item = FECT.allAddons[i];
		if(item.id == target.id) continue;
		if(item.id == FECT.FECT_GUID) continue;
		if(item.appDisabled) continue;
		if(item.userDisabled){
			disabledExt.appendItem(item.name,item.id);
			FECT.contDisabledList.push(item);
			disCnt++;
		}else{
			enabledExt.appendItem(item.name,item.id);
			FECT.contEnabledList.push(item);
			enCnt++;
		}
	}
	document.getElementById('enabledCnt').value = enCnt;
	document.getElementById('disabledCnt').value = disCnt;
	return true;
},

clearListBox: function(which){
	// Clear list entries in <which>
	let theList = document.getElementById(which);
	let children = theList.childNodes;
	let n = children.length;	
	for (let i = 0; i < n; i++) {
		theList.removeChild(children[0]);
	}
	return true;
},

createDB: function(){
	let dbSchema = {
		tables: {
			header:		"sessionID				INTEGER PRIMARY KEY NOT NULL UNIQUE,\
								targetName  			TEXT NOT NULL,\
								targetGUID				TEXT NOT NULL,\
								currentIteration	INTEGER,\
								status						TEXT NOT NULL,\
								culprit						TEXT"
			,
								
			iteration:"itSessionID 	INTEGER NOT NULL,\
								iteration 		INTEGER NOT NULL,\
								extName 			TEXT NOT NULL,\
								extGUID 			TEXT NOT NULL,\
								status 				TEXT NOT NULL,\
								PRIMARY KEY ('itSessionID', 'iteration', 'extGUID')"
			,
			
			suspect:  "susSessionID 	INTEGER NOT NULL,\
								susIteration 		INTEGER NOT NULL,\
								susExtName 			TEXT NOT NULL,\
								susExtGUID 			TEXT NOT NULL,\
								PRIMARY KEY ('susSessionID', 'susIteration', 'susExtGUID')"
			,
			
			resolved: "resSessionID 	INTEGER NOT NULL,\
								resIteration 		INTEGER NOT NULL,\
								resolved 				TEXT NOT NULL,\
								PRIMARY KEY ('resSessionID', 'resIteration')"

		}
	}
	
	FECT.openDB();	
	for(let tName in dbSchema.tables){
		FECT.DB.createTable(tName, dbSchema.tables[tName]);
	}
	return true;
},

openDB: function(){
	if(!FECT.DB.connectionReady){
		let wm = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
		let win = wm.getMostRecentWindow("navigator:browser");
		let file = win.FileUtils.getFile('ProfD', [FECT.DBname]);
		FECT.DB = win.Services.storage.openDatabase(file);	
	}
	return true;
},

fixTargetExtName: function(){
	// Unescape target extension name for display (replace two consecutive single quotes with a single quotation mark)
	let tmp = FECT.targetExt;
	tmp = tmp.replace(/\'\'/g,"'");
	return tmp;
},

startSession: function(){
	FECT.sessionID = document.getElementById('FECTsessionId').value;
	FECT.targetExt = document.getElementById('fectTargetExtName').value;
	FECT.targetExt = FECT.targetExt.replace(/\'/g,"''")	// Escape single quote for sqlite db
	let targetGUID = document.getElementById('fectTargetExtID').value;
	FECT.iteration = 0;
	
	// Set preferences
	FECT.fectSetPrefs();
	
	FECT.openDB();
	
	// Write header record into DB
	let stmt = "INSERT INTO header VALUES('"+FECT.sessionID+"', '"+FECT.targetExt+"', '"+targetGUID+"', '"+FECT.iteration+"','In session','')";
	FECT.executeStatement(stmt);

	// Write iteration data to DB
	FECT.buildIterationData('enabled');
	FECT.buildIterationData('disabled');
	
	FECT.fectPrefs.setBoolPref("extensions.fect.conflictResolved", false);	
	
	// Update FECT menu items
	FECT.setMenu(true);

	window.close();
	FECT.openIterationWin();
	return true;
},

buildIterationData: function(status){
	// Write iteration data to DB
	let theList = (status == 'enabled') ? FECT.contEnabledList : FECT.contDisabledList;
	
	for(let i in theList){
		let item = theList[i];
		item.name = item.name.replace(/\'/g,"''")
		let stmt = "INSERT INTO iteration VALUES('"+FECT.sessionID+"', '"+FECT.iteration+"', '"+item.name+"', '"+item.id+"', '"+status+"')";
		FECT.executeStatement(stmt);
	}
	return true;
},

buildSuspectList: function(iteration,status){
	// Write suspect data to DB
	FECT.getCurrentSessionInfo();
	FECT.getExtensionArrays(FECT.iteration);
	let theList = (status == 'enabled') ? FECT.contEnabledList : FECT.contDisabledList;
	for(let i in theList){
		let item = theList[i];
		item.name = item.name.replace(/\'/g,"''")
		let stmt = "INSERT INTO suspect VALUES('"+FECT.sessionID+"', '"+iteration+"', '"+item.name+"', '"+item.id+"')";
		FECT.executeStatement(stmt);
	}
	return true;
},

executeStatement: function(stmt){
	let result;
//FECT.logMsg(stmt)
	try{
		result = FECT.DB.createStatement(stmt);
		FECT.DB.executeSimpleSQL(stmt);
	}catch(e){
		//FECT.logMsg(e)
	}finally{
		result.reset();
		result.finalize();
	}
	return true;
},

resetDatabase: function(){
	// Reset database and set FECT session data to default
	let msg;
	FECT.getCurrentSessionInfo();
	let sessionInProgress = (FECT.sessionID != "") ? true : false;
	if(sessionInProgress){
		let target = FECT.fectPrefs.getCharPref("extensions.fect.currentTargetExt");	
		msg = FECT.fectMsg[11].replace("%TARGETEXT%",target)+"\n\n";
		msg += FECT.fectMsg[12];
		FECT.fectAlert(msg);
		return true;
	}
	msg = FECT.fectMsg[13];
	let ok = FECT.fectConfirm(msg);
	if(!ok) return true;
	
	FECT.openDB();
	function dropTable(element, index, array) {
		let stmt = "DROP TABLE "+element;
		FECT.executeStatement(stmt);
	}
	['header','iteration','suspect','resolved'].forEach(dropTable);

	FECT.fectResetPrefs();
	FECT.createDB();
	msg = FECT.fectMsg[14].replace("%DBNAME%",FECT.DBname);
	FECT.fectAlert(msg);

	return true;
},

openIterationWin: function(){
	FECT.fectSessionWindow = window.openDialog('chrome://fect/content/FECTcontinueSession.xul','','centerscreen,resizable','');
	return true;
},

initContinueSession: function(){
	// This is the onload for FECTcontinueSession.xul
	FECT.fectInitMsgs();
	FECT.openDB();
	FECT.getCurrentSessionInfo();
	let title = FECT.fectMsg[1].replace("%TARGETEXT%",FECT.fixTargetExtName());
	window.document.title = title;
	
	let lastIterationGroup = document.getElementById('lastIterationGroup');
	let nextIterationGroup = document.getElementById('nextIterationGroup');

	if(FECT.iteration == 0){
		lastIterationGroup.hidden = true;
		nextIterationGroup.hidden = false;
		document.documentElement.getButton("extra1").disabled = false;
	}else{
		lastIterationGroup.hidden = false;
		nextIterationGroup.hidden = true;
		document.documentElement.getButton("extra1").disabled = true;
	}
	let d = document.getElementById("lastiterationtitle");
	d.value+= ": "+parseInt(FECT.iteration,10).toString();
	
	// Fill enabled/disabled extensions listboxes
	let id = "fectContEnabledExtList";
	FECT.clearListBox(id);
	let enabledBox = document.getElementById(id);
	
	id = "fectContDisabledExtList";
	FECT.clearListBox(id);
	let disabledBox = document.getElementById(id);

	id = "fectPriorDisabledExtList";
	FECT.clearListBox(id);
	let priorDisabledBox = document.getElementById(id);
	
	FECT.getExtensionArrays(FECT.iteration)
	
	for(let i=0; i<FECT.extensionsArray['enabled'].length; i++){
		let item = FECT.extensionsArray['enabled'][i];
		enabledBox.appendItem(item.name,item.id);
	}
	document.getElementById('enabledCnt').value = FECT.extensionsArray['enabled'].length;
	
	for(let i=0; i<FECT.extensionsArray['disabled'].length; i++){
		let item = FECT.extensionsArray['disabled'][i];
		disabledBox.appendItem(item.name,item.id);
	}
	document.getElementById('disabledCnt').value = FECT.extensionsArray['disabled'].length;

	// Fill prior disabled extensions listbox
	if(FECT.iteration > 0){
		FECT.getExtensionArrays(0);
		document.getElementById('fectPriorDisabledExtList').hidden = false;
		document.getElementById('priorDisabledLbl').hidden = false;
		document.getElementById('priorDisabledCnt').hidden = false;

		for(let i=0; i<FECT.extensionsArray['disabled'].length; i++){
			let item = FECT.extensionsArray['disabled'][i];
			priorDisabledBox.appendItem(item.name,item.id);
		}
		document.getElementById('priorDisabledCnt').value = FECT.extensionsArray['disabled'].length;
	}else{
		document.getElementById('fectPriorDisabledExtList').hidden = true;
		document.getElementById('priorDisabledLbl').hidden = true;
		document.getElementById('priorDisabledCnt').hidden = true;
	}
	sizeToContent();
	if(FECT.iteration == 0) FECT.nextIteration(false);
	return true;
},

nextIteration: function(resolved){
	FECT.fectInitMsgs();
	FECT.openDB();
	FECT.getCurrentSessionInfo();
	
	let lastIterationGroup = document.getElementById('lastIterationGroup');
	let nextIterationGroup = document.getElementById('nextIterationGroup');
	nextIterationGroup.hidden = false;

	let nextIter = FECT.iteration+1;
	let d = document.getElementById("nextiterationtitle");
	d.value = d.value.replace("$ITERATION$"," "+parseInt(nextIter,10).toString());

	// Get list of suspect extensions for the next iteration
	let status = (resolved == 'yes') ? "disabled" : "enabled";
	FECT.getExtensionArrays(FECT.iteration);
	
	// Build suspect list
	let id = "suspectExtList";
	FECT.clearListBox(id);
	let suspectBox = document.getElementById(id);
	let suspectList = [];
	for(let i=0; i<FECT.extensionsArray[status].length; i++){
		let item = FECT.extensionsArray[status][i];
		suspectBox.appendItem(item.name,item.id);
		suspectList.push(item.name);
	}
	document.getElementById('suspectCnt').value = suspectList.length;
		
	// Split the suspect list in half
	FECT.nameGuidXref = FECTstorage.getItem("fectNameGuidXref", "");
	let arrays = FECT.splitArray(suspectList);
	let enabledList = arrays[0];
	let disabledList = arrays[1];
	let enCnt = 0;
	let disCnt = 0;
	
	// Build enabled list
	id = "nextEnabledExtList";
	FECT.clearListBox(id);
	FECT.contEnabledList = [];
	let enabledBox = document.getElementById(id);
	for(let i=0; i<enabledList.length; i++){
		let item = {};
		item.name = enabledList[i];
		item.id = FECT.nameGuidXref[item.name];
		enabledBox.appendItem(item.name,item.id);
		FECT.contEnabledList.push(item);
		enCnt++;
	}
	document.getElementById('nextEnabledCnt').value = enCnt;
	
	// Build disabled list
	id = "nextDisabledExtList";
	FECT.clearListBox(id);
	FECT.contDisabledList = [];
	let disabledBox = document.getElementById(id);
	for(let i=0; i<disabledList.length; i++){
		let item = {};
		item.name = disabledList[i];
		item.id = FECT.nameGuidXref[item.name];
		disabledBox.appendItem(item.name,item.id);
		FECT.contDisabledList.push(item);
		disCnt++;
	}
	document.getElementById('nextDisabledCnt').value = disCnt;
	sizeToContent();
	return true;
},

continueSession: function(){	
	let msg = FECT.fectMsg[7];
	let ok = FECT.fectConfirm(msg);
	if(!ok){
		msg = FECT.fectMsg[8]+"\n\n";
		msg += FECT.fectMsg[9];
		FECT.fectAlert(msg);
		return true;
	}
	
	FECT.iteration++;
	FECT.fectPrefs.setIntPref("extensions.fect.currentIteneration",FECT.iteration);
	
	// Update db with this iteration's configuration
	FECT.openDB();
	FECT.getCurrentSessionInfo();
	
	// Enable extensions for this iteration
	for(let i in FECT.contEnabledList){
		let item = FECT.contEnabledList[i];
		AddonManager.getAddonByID(item.id, function(addon){
			addon.userDisabled = false;
		});
	}
		
	// Disable extensions for this iteration
	for(let i in FECT.contDisabledList){
		let item = FECT.contDisabledList[i];
		AddonManager.getAddonByID(item.id, function(addon){
			addon.userDisabled = true;
		});
	}

	// Update currentIteration in header table
	stmt = "UPDATE header SET currentIteration="+FECT.iteration+" WHERE sessionId="+FECT.sessionID;
	result = FECT.DB.createStatement(stmt);
	FECT.DB.executeSimpleSQL(stmt);
	
	// Update iteration table
	FECT.buildIterationData('enabled');
	FECT.buildIterationData('disabled');
	
	// Update suspect table
	let resolved = FECT.fectPrefs.getBoolPref("extensions.fect.conflictResolved");
	let status = (resolved) ? "disabled" : "enabled";
	FECT.buildSuspectList(FECT.iteration,status);
	
	FECT.fectPrefs.setIntPref("extensions.fect.currentIteneration",FECT.iteration);
	window.close();
	FECT.fectRestartFx();
	return true;
},

cancelSession: function(){
	FECT.getCurrentSessionInfo();
	let msg;
	if(FECT.iteration == 0){
	
		// Remove session header record
		let stmt = "DELETE FROM header WHERE sessionID = '"+FECT.sessionID+"'";
		let result = FECT.DB.createStatement(stmt);
		FECT.DB.executeSimpleSQL(stmt);

		// Remove iteration records
		stmt = "DELETE FROM iteration WHERE itSessionID = '"+FECT.sessionID+"'";
		result = FECT.DB.createStatement(stmt);
		FECT.DB.executeSimpleSQL(stmt);
		
		// Reset preferences and menu
		FECT.fectResetPrefs();
		FECT.setMenu(false);
		
		msg = FECT.fectMsg[18].replace("%SESSIONID%",FECT.sessionID)
		msg = msg.replace("%TARGETEXT%",FECT.fixTargetExtName());
	}else{
		msg = FECT.fectMsg[19].replace("%NEXTITERATION%",FECT.iteration+1);
		msg = msg.replace("%CURRENTITERATION%",FECT.iteration)+"\n";
		msg += FECT.fectMsg[20].replace("%TARGETEXT%",FECT.fixTargetExtName());
	}
	
	FECT.fectAlert(msg);
	window.close();
	return true;
},

resolvedStatus: function(){
	let resolved = document.getElementById('fectConflictResolved').value;
	FECT.resolved = (resolved == 'yes') ? true : false;
	FECT.fectPrefs.setBoolPref("extensions.fect.conflictResolved", FECT.resolved);	
	
	let enabledCnt = document.getElementById('enabledCnt').value;
	let disabledCnt = document.getElementById('disabledCnt').value;
	let msg;
	if(enabledCnt == 0 && disabledCnt == 1){
		if(FECT.resolved){
			let theList = document.getElementById("fectContDisabledExtList");
			let item = theList.getItemAtIndex(0);
			let culprit = item.label;
			msg = FECT.fectMsg[21].replace("%TARGETEXT%",FECT.fixTargetExtName());
			msg = msg.replace("%CULPRIT%",culprit);
			msg += "\n\n"+FECT.fectMsg[22]+"\n\n"+FECT.fectMsg[23];
			FECT.fectAlert(msg);
			
			FECT.openDB();
			let stmt = "UPDATE header SET status='Resloved', culprit='"+culprit+"' WHERE sessionId="+FECT.sessionID;
			let result = FECT.DB.createStatement(stmt);
			FECT.DB.executeSimpleSQL(stmt);
			FECT.fectMainWindow.FECT.fectSessionWindow.close();
		}else{
			msg = FECT.fectMsg[24];
			msg += "\n\n"+FECT.fectMsg[23];
			FECT.fectAlert(msg);
			
			FECT.openDB();
			let stmt = "UPDATE header SET status='Unresloved', culprit='Unknown' WHERE sessionId="+FECT.sessionID;
			let result = FECT.DB.createStatement(stmt);
			FECT.DB.executeSimpleSQL(stmt);
			FECT.fectMainWindow.FECT.fectSessionWindow.close();
		}
		return true;
	}
	FECT.openDB();
	let stmt = "SELECT * FROM resolved WHERE resSessionID='"+FECT.sessionID+"' AND resIteration='"+FECT.iteration+"'";
	let result = FECT.DB.createStatement(stmt);
	if(result.executeStep()){
		stmt = "UPDATE resolved SET resolved='"+FECT.resolved+"' WHERE resSessionID='"+FECT.sessionID+"'";		
		result = FECT.DB.createStatement(stmt);
		FECT.DB.executeSimpleSQL(stmt);
	}else{
		stmt = "INSERT INTO resolved VALUES('"+FECT.sessionID+"','"+FECT.iteration+"','"+FECT.resolved+"')";	
		result = FECT.DB.createStatement(stmt);
		FECT.DB.executeSimpleSQL(stmt);
	}
			
	FECT.nextIteration(resolved);
	document.documentElement.getButton("extra1").disabled = false;
},

resolvedPicked: function(){
	let selected = document.getElementById("fectConflictResolved").selectedIndex;
	let resolvedBtn = document.getElementById('reslolvedBtn');
	if(selected == 2){
		resolvedBtn.disabled = true;
	}else{
		resolvedBtn.disabled = false;
	}
},

stripped: function(name){
	// Strip spaces and convert to lowercase
	let newname = name.toLowerCase(); 
	print(newname)
	return newname.replace(/\s+/g, '');
},

getExtensionArrays: function(iteration){
//FECT.logMsg("Getting extensionarrays for iteration "+iteration)
	// Retrieve from the db a list of all enabled/disabled/suspect extensions for a particular iteration
	FECT.extensionsArray = {};
	FECT.extensionsArray['enabled'] = []; 
	FECT.extensionsArray['disabled'] = [];
	FECT.extensionsArray['suspect'] = [];
	
	let enabledNameArray = [];
	let disabledNameArray = [];
	let suspectNameArray = [];
	let xRef = {};
	let stmt,result;
	
	stmt = "SELECT * FROM iteration WHERE itSessionID='"+FECT.sessionID+"' AND iteration='"+iteration+"'";
	result = FECT.DB.createStatement(stmt);
		
	try{
		while (result.executeStep()) {
			let status = result.row.status;
			let item = {};
			item.name = result.row.extName;
			item.guid = result.row.extGUID;
			if(status == "enabled"){
				enabledNameArray.push(item.name);
			}else{
				disabledNameArray.push(item.name);
			}
			xRef[item.name] = item;
		};
	}finally{
		result.reset();
	}
	result.finalize();
	
	enabledNameArray.sort();
	disabledNameArray.sort();
	
	for(let i=0; i<enabledNameArray.length; i++){
		let item = xRef[enabledNameArray[i]]
		FECT.extensionsArray['enabled'].push(item);
	}
	
	for(let i=0; i<disabledNameArray.length; i++){
		let item = xRef[disabledNameArray[i]]
		FECT.extensionsArray['disabled'].push(item);
	}

	stmt = "SELECT * FROM suspect WHERE susSessionID='"+FECT.sessionID+"' AND susIteration='"+iteration+"'";
	result = FECT.DB.createStatement(stmt);
		
	try{
		while (result.executeStep()) {
			let item = {};
			item.name = result.row.susExtName;
			item.guid = result.row.susExtGUID;
			let sortName = FECT.stripped(item.name);
			suspectNameArray.push(sortName);
			xRef[sortName] = item;
		};
	}finally{
		result.reset();
	}
	result.finalize();
	
	suspectNameArray.sort();
	
	for(let i=0; i<suspectNameArray.length; i++){
		let item = xRef[suspectNameArray[i]]
		FECT.extensionsArray['suspect'].push(item);
	}
	
	return true;
},

splitArray: function(array){
	//  Randomly split array in half
	let half = parseInt(array.length/2,10); 
	let arrays = [];
	arrays[0] = [];
	arrays[1] = [];
	let workArray = vacuum(array);
	for(let i=0; i <= half-1; i++){
		let len = workArray.length;
		let which = Math.floor((Math.random() * len)); 
		arrays[0].push(workArray[which])
		delete workArray[which];
		workArray = vacuum(workArray);
	}
	arrays[0].sort(caseInsensitive);
	arrays[1] = vacuum(workArray);
	return arrays;
	
	function caseInsensitive(a, b) {
		return a.toLowerCase().localeCompare(b.toLowerCase());  
	};

	function vacuum(array){
		// Remove deleted entries
		let tmp = [];
		for(let i=0; i<array.length; i++){
			if(typeof array[i] != "undefined") tmp.push(array[i]);
		}
		return tmp;
	}
},

displaySessionInfo: function(){
	FECT.getCurrentSessionInfo();
	let msg;
	if(FECT.sessionID != ""){
		msg = FECT.fectMsg[15].replace("%SESSIONID%",FECT.sessionID);
		msg = msg.replace("%ITERATION%",FECT.iteration)+"\n\n";
		msg += FECT.fectMsg[16].replace("%EXTNAME%",FECT.fixTargetExtName());
	}else{
		msg = FECT.fectMsg[17];
	}
	FECT.fectAlert(msg);
	return true;
},

displayProgressReport: function(){
	FECT.fectInitMsgs();
	FECT.openDB();
	FECT.getCurrentSessionInfo();
	
  // Get results template 
	let resultsTemplate=FECT.getReportTemplate("FECTprogressReport.html");

	// Put the template into a DOM parser
	let domParser = Cc["@mozilla.org/xmlextras/domparser;1"].createInstance(Ci.nsIDOMParser);
	let doc = domParser.parseFromString(resultsTemplate, "text/html");
	doc.title = FECT.fectMsg[2];
			
  function caseInsensitive(a, b) {
		return a.toLowerCase().localeCompare(b.toLowerCase());  
	};
		
	// Local variables
	let re = new RegExp("\\\\", "g");
	let len = 0;
	let errCnt = 0;
	let warnCnt = 0;
	let resultsPageURL = "";
	let txt, elm, txtNode, table, tbody, tr, td, msg, tmp, item, div, span;
	let stmt, result, caption;

	// Element palette
	//let body = document.getElementsByTagName('body')[0];
	let br = document.createElementNS(FECT.fectNamespace,'br');
	let captionElm = document.createElementNS(FECT.fectNamespace,'caption');
	let cellElm = document.createElementNS(FECT.fectNamespace,'td');
	let cellStyle = 'text-align: center;';
	let imgElm = document.createElementNS(FECT.fectNamespace,'img');
	let divElm = document.createElementNS(FECT.fectNamespace,'div');
	let pElm = document.createElementNS(FECT.fectNamespace,'p');
	let rowElm = document.createElementNS(FECT.fectNamespace,'tr');
	let spanElm = document.createElementNS(FECT.fectNamespace,'span');
	let tableElm = document.createElementNS(FECT.fectNamespace,'table');
	let theadElm = document.createElementNS(FECT.fectNamespace,'thead');
	let thElm = document.createElementNS(FECT.fectNamespace,'th');
	let tfootElm = document.createElementNS(FECT.fectNamespace,'tfoot');
	let tbodyElm = document.createElementNS(FECT.fectNamespace,'tbody');
	let trElm = document.createElementNS(FECT.fectNamespace,'tr');
	let tdElm = document.createElementNS(FECT.fectNamespace,'td');
	let anchorElm = document.createElementNS(FECT.fectNamespace,'a');
	let mdash = " "+String.fromCharCode(8212)+" ";
	let body = doc.body;
	let divider = "  ";
	for(let i=1; i<51; i++) divider += "~";
			
	// Report title
	elm = doc.getElementById("reportheading");
	txt = document.createTextNode(FECT.fectMsg[2]);
	elm.appendChild(txt);
		
	// Session start
	elm = doc.getElementById("startsession");
	tmp = "Session %ID% started on %DATE% with target extension: %TARGET%";
	let dte = FECT.fectLocalizedDate(FECT.sessionID);
	tmp = tmp.replace('%DATE%',dte);
	tmp = tmp.replace('%TARGET%',FECT.fixTargetExtName());
	tmp = tmp.replace('%ID%',FECT.sessionID);
	
	txt = document.createTextNode(tmp);
	elm.appendChild(txt);
	elm.appendChild(br.cloneNode(false));
	elm.appendChild(br.cloneNode(false));
	
	FECT.getExtensionArrays(0);
	
	// Enabled/disabled extension tables for start of session
	makeTable('Enabled');
	makeTable('Disabled');
	
	// Iterations
	stmt = "SELECT * FROM header WHERE sessionID='"+FECT.sessionID+"'";
	let headerRecord = FECT.DB.createStatement(stmt);
	if(!headerRecord.executeStep()){
		tmp = FECT.fectMsg[10];
		FECT.fectAlert(tmp);
		return true;
	}
	let currentIteration = parseInt(headerRecord.row.currentIteration,10);
	for(let i=1; i<=currentIteration; i++){
		FECT.getExtensionArrays(i);
		elm = doc.getElementById("iterations");
		tmp = "Iteration: "+i.toString();
		if(i == currentIteration){
			tmp += "  (Current iteration)";
		}else{
			stmt = "SELECT * FROM resolved where resIteration='"+i+"' AND resSessionId='"+FECT.sessionID+"'";
			result = FECT.DB.createStatement(stmt);
			result.executeStep();
			let resolved = result.row.resolved;
			if(resolved){
				tmp += "  "+FECT.fectMsg[26]; //" - Conflict was resolved";	
			}else{
				tmp += "  "+FECT.fectMsg[27]; //" - Conflict was NOT resolved";
			}
		}
		txt = document.createTextNode(tmp);
		elm.appendChild(txt);
			
		txt = document.createTextNode(divider);
		span = spanElm.cloneNode(false);
		span.className = "divider";
		span.appendChild(txt);
		elm.appendChild(span)
	
		elm.appendChild(br.cloneNode(false));
		elm.appendChild(br.cloneNode(false));
		
		makeTable('Enabled');
		makeTable('Disabled');
		makeTable('Suspect');
	}
	
	let culprit = headerRecord.row.culprit;
	if(culprit != ""){
		elm = doc.getElementById("conclusion");
		elm.appendChild(br.cloneNode(false));
		tmp = FECT.fectMsg[25];
		tmp = tmp.replace("%TARGETEXT%",FECT.fixTargetExtName());
		tmp = tmp.replace("%CULPRIT%",culprit);
		txt = document.createTextNode(tmp);
		elm.appendChild(txt)
	}

  // Write content to results page
	let fectProgressPage = FECT.fectProfDir.clone();
	fectProgressPage.append("FECTprogress.html");
	
	let resultsFile = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
	resultsFile.init(fectProgressPage, 0x02 | 0x08 | 0x20, 0x1ED, 0); // write, create, truncate
	let pageSource = doc.documentElement.outerHTML;
	
	let charset = "UTF-8";
	let os = Cc["@mozilla.org/intl/converter-output-stream;1"].createInstance(Ci.nsIConverterOutputStream);

	os.init(resultsFile, charset, 4096, 0x0000);
	os.writeString(pageSource);
	os.close();
	FECT.fectOpenProgressPage(fectProgressPage.path);

	function makeTable(which){
		let arrIndex = which.toLowerCase();
		let table = tableElm.cloneNode(false);
		table.setAttribute("class","table");
		let caption = captionElm.cloneNode(false);
		let len = FECT.extensionsArray[arrIndex].length;
		let tmp = which+" extensions ("+len+"): ";
		let txt = document.createTextNode(tmp);
		caption.appendChild(txt);
		table.appendChild(txt);
		
		let tbody = tbodyElm.cloneNode(false);
		let numCols = 5;
		let numRows = 0;
		if(len <= numCols){
			numCols = len;
			numRows = 1;
		}else{
			numRows = parseInt(len/numCols,10)+1;
		}
		let itemNum = 0;
		for(let row=1; row<=numRows; row++){
			tr = trElm.cloneNode(false);
			for(let col=1; col<=numCols; col++){
				td = tdElm.cloneNode(false);
				let item = FECT.extensionsArray[arrIndex][itemNum];
				if(!!item){
	//FECT.propsConsole(item,"item",false,true,true,true,true)
					tmp = item.name;
					txt = document.createTextNode(tmp);
				}else{
					txt = br.cloneNode(false);
				}
				td.appendChild(txt);
				tr.appendChild(td);
				itemNum++;
			}
			tbody.appendChild(tr);
		}
		table.appendChild(tbody);
		elm.appendChild(table);
		elm.appendChild(br.cloneNode(false));
		elm.appendChild(br.cloneNode(false));
		return true;
	}
	
	return true;
/*	
	for(let i=0; i<len; i++){
		let item = FECT.extensionsArray['enabled'][i];
		tmp += item.name;
		if(i != len-1) tmp += ", ";
	}
	elm.appendChild(br.cloneNode(false));
	elm.appendChild(br.cloneNode(false));

	len = FECT.extensionsArray['disabled'].length;
	tmp = "Disabled extensions ("+len+"): ";
	for(let i=0; i<len; i++){
		let item = FECT.extensionsArray['disabled'][i];
		tmp += item.name;
		if(i != len-1) tmp += ", ";
	}
	txt = document.createTextNode(tmp);
	elm.appendChild(txt);
	elm.appendChild(br.cloneNode(false));
	elm.appendChild(br.cloneNode(false));
	
	// Iterations
	stmt = "SELECT * FROM header WHERE sessionID='"+FECT.sessionID+"'";
	result = FECT.DB.createStatement(stmt);
	if(!result.executeStep()){
		tmp = FECT.fectMsg[10];
		FECT.fectAlert(tmp);
		return true;
	}
	let currentIteration = parseInt(result.row.currentIteration,10);
	for(let i=1; i<=currentIteration; i++){
		elm = doc.getElementById("iterations");
		tmp = "Iteration: "+i.toString();
		if(i == currentIteration) tmp += "  (Current iteration)";
		txt = document.createTextNode(tmp);
		elm.appendChild(txt);
		elm.appendChild(br.cloneNode(false));
		elm.appendChild(br.cloneNode(false));
		
		// Suspect extensions
		FECT.getExtensionArrays(i);
		len = FECT.extensionsArray['suspect'].length;
		tmp = "Suspect extensions: ("+len+"): ";
		
		for(let j=0; j<len; j++){
			let item = FECT.extensionsArray['suspect'][j];
			tmp += item.name;
			if(j != len-1) tmp += ", ";
		}
		txt = document.createTextNode(tmp);
		elm.appendChild(txt);
		elm.appendChild(br.cloneNode(false));
		elm.appendChild(br.cloneNode(false));
	}
	
	len = FECT.extensionsArray['disabled'].length;
	tmp = "Disabled extensions ("+len+"): ";
	for(let i=0; i<len; i++){
		let item = FECT.extensionsArray['disabled'][i];
		tmp += item.name;
		if(i != len-1) tmp += ", ";
	}
	txt = document.createTextNode(tmp);
	elm.appendChild(txt);
	elm.appendChild(br.cloneNode(false));
	elm.appendChild(br.cloneNode(false));
	
	len = FECT.extensionsArray['enabled'].length;
	tmp = "Enabled extensions ("+len+"): ";
	for(let i=0; i<len; i++){
		let item = FECT.extensionsArray['enabled'][i];
		tmp += item.name;
		if(i != len-1) tmp += ", ";
	}
	txt = document.createTextNode(tmp);
	elm.appendChild(txt);
	elm.appendChild(br.cloneNode(false));
	elm.appendChild(br.cloneNode(false));


	
	//*************************************
	function buildLineItem(cnt,item){
		let tr, td, num, img, link, span, name, ver, desc;
		tr = trElm.cloneNode(false);
		
		// Count
		td = tdElm.cloneNode(false);
		td.className = "num";
		num = document.createTextNode(cnt+".");
		td.appendChild(num);
		tr.appendChild(td);
		
		// Icon
		td = tdElm.cloneNode(false);
		td.className = "top";
		img = imgElm.cloneNode(false);
		img.className = "icon";
		img.src = item.iconURL;
		td.appendChild(img);
		tr.appendChild(td);
		
		// Name/link
		td = tdElm.cloneNode(false);
		if(item.homepageURL != null){
			link = anchorElm.cloneNode(false);
			link.href = item.homepageURL;
			link.textContent = item.name;
			link.className = "itemname";
			if(item.userDisabled) link.className += " disabled";
			if(typeof item.tooltipdata == undefined) item.tooltipdata = "";
			link.setAttribute('data-tooltipdata',item.tooltipdata);
			td.appendChild(link);
		}else{
			span = spanElm.cloneNode(false);
			span.className = "itemname";
			if(item.userDisabled) span.className += " disabled";
			name = document.createTextNode(item.name);
			span.appendChild(name);
			if(typeof item.tooltipdata === undefined) item.tooltipdata = "";
			span.setAttribute('data-tooltipdata',item.tooltipdata);
			td.appendChild(span);
		}	
		
		// Version
		span = spanElm.cloneNode(false);
		span.className = "ver";
		ver = document.createTextNode(" (v"+item.version+")");
		span.appendChild(ver);
		if(item.version != "") td.appendChild(span);
		
		// Description				
		span = spanElm.cloneNode(false);
		span.className = "description";
		txt = "";
		//if(item.isPersona) txt = mdash+FECT.fectMsg[340];
		txt += mdash+item.desc;
		desc = document.createTextNode(txt);
		span.appendChild(desc);
		td.appendChild(span);
		
		tr.appendChild(td);
		return tr;
	}
	
	function buildLineItem2(cnt,item){
		let label = item.desc;
		let fname = item.fname;
		let tr = trElm.cloneNode(false);
		let td, num, span;
		
		// Count
		td = tdElm.cloneNode(false);
		td.className = "num";
		num = document.createTextNode(cnt+".");
		td.appendChild(num);
		tr.appendChild(td);
		
		// Label and fname
		td = tdElm.cloneNode(false);
		txt = document.createTextNode(label+" ");
		td.appendChild(txt);
		span = spanElm.cloneNode(false);
		span.className = "itemname";
		txt = document.createTextNode(fname);
		span.appendChild(txt);
		td.appendChild(span);
			
		tr.appendChild(td);
		return tr;
	}

	function buildLineItem3(cnt,msg){
		let td, num;
		let tr = trElm.cloneNode(false);
		
		// Count
		td = tdElm.cloneNode(false);
		td.className = "num";
		num = document.createTextNode(cnt+".");
		td.appendChild(num);
		tr.appendChild(td);
		
		// Message
		td = tdElm.cloneNode(false);
		td.className = "err";
		txt = document.createTextNode(msg);
		td.appendChild(txt);			
		tr.appendChild(td);
		return tr;
	}
*/
},

fectLocalizedDate: function (aDate){
	let lDate = new Date(parseInt(aDate));
	let rv = lDate.toLocaleDateString()+" "+lDate.toLocaleTimeString();
	if(rv.indexOf("Invalid") != -1){rv = aDate;}
	return rv; 
},

getReportTemplate: function (fName){
	// Returns the contents (text) of fName
	FECT.fectGetPrefs();
	let dFile = FECT.fectInstallDirectory.clone();
	if (!dFile) return false;
	dFile.append("chrome");
	dFile.append("content");
	dFile.append(fName);
	let fis = Cc["@mozilla.org/network/file-input-stream;1"].createInstance(Ci.nsIFileInputStream);
	fis.init(dFile, 0x01, 0x124, 0);
	fis.QueryInterface(Ci.nsILineInputStream);
	let cis = Cc["@mozilla.org/intl/converter-input-stream;1"].createInstance(Ci.nsIConverterInputStream);
	cis.init(fis,"UTF-8", 0, 0x0000);
	let lis = cis.QueryInterface(Ci.nsIUnicharLineInputStream);
	let line = {}, hasmore;	
	let data, content = "";
	do {
		hasmore = lis.readLine(line);
		data = line.value;
		content += data+"\n";
	} while(hasmore);
	cis.close();
	fis.close();
	return content;
},

fectOpenProgressPage: function (URL){
	if(!URL){return false;}
	window.openNewTabWith(URL, this.docURL, null, null);
	return true;
},

endSession: function(){
	FECT.getCurrentSessionInfo();
	let msg = FECT.fectMsg[3].replace('%SESSIONID%',FECT.sessionID);
	msg = msg.replace('%EXTNAME%',FECT.fixTargetExtName());
	msg += "\n\n"+FECT.fectMsg[4];
	msg += "\n\n"+FECT.fectMsg[5];
	let ok = FECT.fectConfirm(msg);
	if(!ok){
		msg = FECT.fectMsg[6].replace('%SESSIONID%',FECT.sessionID);
		FECT.fectAlert(msg);
		return true;
	}
	FECT.openDB();
	let stmt = "SELECT * FROM iteration WHERE itSessionID='"+FECT.sessionID+"' AND iteration='0'";
	let result = FECT.DB.createStatement(stmt);
	try{
		while (result.executeStep()) {
			let guid = result.row.extGUID;
			let status = result.row.status;
			let bool = (status == 'disabled') ? true : false;
	AddonManager.getAddonByID(guid, function(addon){
		addon.userDisabled = bool;
	});

		}
	}finally{
		result.reset();
	}
	result.finalize();
	
	stmt = "UPDATE header SET status='Ended'  WHERE sessionId="+FECT.sessionID;
	result = FECT.DB.createStatement(stmt);
	FECT.DB.executeSimpleSQL(stmt);
	
	FECT.fectResetPrefs();
	
	FECT.fectRestartFx();
	return true;
},

fectRestartFx: function (){
	a = Ci.nsIAppStartup,Cc["@mozilla.org/toolkit/app-startup;1"].getService(a).quit(a.eRestart | a.eAttemptQuit);
	return true;
},


test: function(){
	AddonManager.getAddonByID("{4BBDD651-70CF-4821-84F8-2B918CF89CA3}", function(addon){
		//FECT.propsConsole(addon,"FEBE");
		addon.userDisabled = true;
	});

},

fectOpenLink: function (URL){
/*
	if(!URL){return;}
	let wm = Cc["@mozilla.org/appshell/window-mediator,1"]
           .getService(Ci.nsIWindowMediator);
	let win = wm.getMostRecentWindow("navigator:browser");
	let pasteemailGetBrowser = win.getBrowser();
	let currBlank = (pasteemailGetBrowser &&
	(pasteemailGetBrowser.mCurrentTab.linkedBrowser &&
	(pasteemailGetBrowser.mCurrentTab.linkedBrowser.contentDocument.location == "about:blank")) ||
	(!pasteemailGetBrowser.mCurrentTab.linkedBrowser &&
	(pasteemailGetBrowser.mCurrentTab.label == "(Untitled)")));
	if(currBlank){
	
	let resultsWindow = win.loadURI(URL);
	}else{
		let resultsWindow = win.openNewTabWith(URL, this.docURL, null, null);
  }
	*/
  return true;
},

setFECTVersion: function (){
/*
	FECT.pasteemailInitMsgs();
	let pePref = "extensions.pasteemail.version";
	let peVersion = FECT.pePrefs.getCharPref(pePref);
	document.getElementById("peVersionID").setAttribute("value", FECT.pasteemailMsg[2]+" "+peVersion);
*/
	return true;
},

fectHelp: function (helpIndex,fieldName){
	let opts = FECT.fectSetWinOpts(false);
	FECT.fectWin = window.openDialog('chrome://fect/content/FECThelp.xul', 'FECT Help', opts,helpIndex,fieldName);
	FECT.fectWin.focus();
	return true;
},

fectAlert: function (message,modal){
	FECT.fectGetPrefs();
	let opts = FECT.fectSetWinOpts(modal);	
	FECT.fectWin = FECT.fectMainWindow.openDialog('chrome://fect/content/FECTalert.xul','FECT Alert',opts,message);	
	FECT.fectWin.focus();
	return true;
},

fectConfirm: function(message){
	FECT.fectGetPrefs();
	FECT.fectInitMsgs();
	FECT.getCurrentSessionInfo();
	let opts = FECT.fectSetWinOpts(true);
	FECT.fectWin = FECT.fectMainWindow.openDialog('chrome://fect/content/FECTconfirm.xul','FECT Alert',opts,message);	
	FECT.fectWin.focus();
	return FECT.mainWin.FECT.fectPrefs.getBoolPref('extensions.fect.confirm');
},


fectConfirmOK: function (){
	FECT.fectPrefs.setBoolPref('extensions.fect.confirm',true);
	return true;
},

fectConfirmCancel: function (){
	FECT.fectPrefs.setBoolPref('extensions.fect.confirm',false);
	return true;
},

fectSetAlertMsg: function (){
	// If the message contains a style or class, it must be the first part of the message 
	// (one style per line, if needed).  
	// If a class is assigned, it must be in the form of: <class>classname</class>
	let tmp = window.arguments[0]+"   \n\n";
	let lines = tmp.split(/\n/); // Break it into an array
	let msg = [], re, a;
	let separator = document.createElement('separator');
	for(let i=0;i<lines.length;i++){	
		let line = lines[i];
		let item = {};
		item.Style = "font-family: arial; font-size: 10pt; font-weight: normal; color: purple";
		item.Text = line;

		// Parse for links
		let link = line.indexOf("<link>");
		if(link == 0){
			re = /(<link>)(.*),(.*)(<\/link>)/g
			a = re.exec(line);
			item.Text = a[2];
			item.Link = a[3];
			item.Klass = "link";	
			msg[i] = item;
			continue;
		}
		
		// Parse for styles
		let style = line.indexOf("<style>");
		if(style == 0){
			re = /(<style>)(.*)(<\/style>)(.*)/g
			a = re.exec(line);
			item.Style = a[2];
			item.Text = a[4];
			msg[i] = item;
			continue;
		}
		
		// Parse for class
		let klass = line.indexOf("<class>");
		if(klass == 0){
			re = /(<class>)(.*)(<\/class>)(.*)/g
			a = re.exec(line);
			item.Klass = a[2];
			item.Text = a[4];
			msg[i] = item;
			continue;
		}
		
		msg[i] = item;
	}
	FECT.fectClipboard = [];
	let msgBox = document.getElementById("fectMsgBox");
	if(msgBox){
		for(let i=0; i<msg.length; i++){
			if(msg[i].Text == ""){
				msgBox.appendChild(separator.cloneNode(false));
				FECT.fectClipboard.push(msg[i].Text);
				continue;
			}
			let desc = document.createElement('description');
			desc.setAttribute("style",msg[i].Style);
			if(msg[i].Klass){
				desc.removeAttribute("style");
				desc.setAttribute("class",msg[i].Klass);
				if(msg[i].Klass == "link"){
					let txt = document.createElement('text');
					txt.setAttribute("value",msg[i].Text);
					txt.removeAttribute("onclick");
					clickify(txt, msg[i].Link);
					desc.appendChild(txt);
				}else{
					desc.textContent = msg[i].Text;			
				}
			}else{
				desc.textContent = msg[i].Text;			
			}
			msgBox.appendChild(desc);
			FECT.fectClipboard.push(msg[i].Text);
		}
		try{
			document.documentElement.getButton("cancel").focus();
		}catch(err){;;}
	}
	FECT.fectPlaySound('message');
	
	function clickify(elem, address) {
    elem.setAttribute('href', address);
		elem.onclick = function(event){FECT.fectOpenLink(this.getAttribute('href'))};
		return true;
	}
	return true;
},

fectSetWinOpts: function(modal){
	let opts = "chrome,dialog,alwaysRaised,resizable";
	if(modal){opts += ",modal";}
	let fectMsgType = FECT.fectPrefs.getCharPref('extensions.fect.alertType');
	if(fectMsgType == 'slider'){opts += ",popup";}
	if(fectMsgType == 'standard'){opts += ",centerscreen,titlebar";}
	return opts;
},

fectSliderLoad: function (noSlide){
	let fectMsgType = FECT.fectPrefs.getCharPref('extensions.fect.alertType');
	
	// 'noSlide' is for resizable windows opened with window.openDialog()
	if(typeof noSlide == 'undefined') noSlide = false;
	if(noSlide) fectMsgType = "standard";
	sizeToContent();
	let finalHeight = window.outerHeight;
	let windowtype = document.documentElement.getAttribute("windowtype");

	if(fectMsgType == 'slider'){
		var sliderIncrement = 10;		// Amount the slider grows by (pixels)
		var sliderTime = 25;				// Time between increments (milliseconds)
		window.outerHeight = 1;			// First shrink the window down
		let x =(screen.availLeft + screen.availWidth - window.outerWidth) - 10;
		if(windowtype == 'fect:help') x=0;	// Help screens popup on the left
		let y = screen.availTop + screen.availHeight - window.outerHeight;
		window.moveTo(x,y);
		window.focus();
		setTimeout(function(){animateAlert();}, sliderTime);
	}
	function animateAlert(){
		if(window.outerHeight < finalHeight){
			window.screenY -= sliderIncrement;
			window.outerHeight += sliderIncrement;
			setTimeout(function(){animateAlert()}, sliderTime);
			return true;
		}
	}
	
	// On windows that don't have a 'Copy to clipboard' button, the button label will be blank
	let copy2clipboardBtn = document.documentElement.getButton("extra1");
	let copy2clipboard = FECT.fectPrefs.getBoolPref("extensions.fect.copyToClipboard");
	copy2clipboardBtn.hidden = (!copy2clipboard || (copy2clipboardBtn.label == ""));
	try{
		document.documentElement.getButton("cancel").focus();
	}catch(err){;;}
	return true;
},

fectCopyToClipboard: function (){
  // Copy the contents of a fectAlert, fectConfirm, or fectError box to the system clipboard
	let gClipboardHelper = Cc["@mozilla.org/widget/clipboardhelper;1"].getService(Ci.nsIClipboardHelper);
	gClipboardHelper.copyString(FECT.fectClipboard.join("\n"));
	FECT.fectClipboard = [];
	FECT.fectPlaySound('success');
	return true;
},

fectPlaySound: function (which){
	FECT.fectGetPrefs();
	FECT.fectHandleSounds();
	if(!FECT.fectPlaySounds) return true;
	let soundURL = FECT.fectPlaySoundsURI[which];	
	let audio = new Audio(soundURL); 
	audio.play();
	return true;
},

fectHandleSounds: function (){
	FECT.fectPlaySounds = FECT.fectPrefs.getBoolPref("extensions.fect.playSounds");	
	FECT.fectPlaySoundsURI['message'] = "chrome://fect/content/sounds/alert.ogg";	
	FECT.fectPlaySoundsURI['help'] = "chrome://fect/content/sounds/help.ogg";	
	FECT.fectPlaySoundsURI['failure'] = "chrome://FECT/content/sounds/failure.ogg";
	FECT.fectPlaySoundsURI['success'] = "chrome://FECT/content/sounds/success.ogg";
	return true;
},

// FECT Help routines
fectLoadHelp: function (){
	var index = window.arguments[0];
	var field = window.arguments[1];
	FECT.fectClipboard = [];

	document.getElementById("field").value = field;
	var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"].getService(Ci.nsIStringBundleService);
	var helpFile = stringBundleService.createBundle("chrome://fect/locale/FECThelp.properties");
	var helpTxt = helpFile.GetStringFromName("fect.help_"+index+".text1");
	
	// XUL palette
	var description = document.createElement('description');
	description.setAttribute('flex', "1");
	var separator = document.createElement('separator');
	separator.setAttribute('class', "thin");
	var vbox = document.createElement('vbox');
	var hbox = document.createElement('hbox');
	var grid = document.createElement('grid');
	var columns = document.createElement('columns');
	var column = document.createElement('column');
	var rows = document.createElement('rows');
	var row = document.createElement('row');
	var spacer = document.createElement('spacer');
	spacer.setAttribute('flex', "1");
	var image = document.createElement('image');
	var label = document.createElement('label');
	var text = document.createElement('text');
	var htmlul = document.createElementNS(this.fectNamespace,'html:ul');
	var htmlli = document.createElementNS(this.fectNamespace,'html:li');
	var htmlbr = document.createElementNS(this.fectNamespace,'html:br');
	var helpTxtbox = document.getElementById('helptextbox');
	switch(index){

		default:
			addHelpLines(index);
			break;
	}//switch
	
	function addHelpLines(index){
		var line = 1;
		while(true){
			try{
				var helpTxt = helpFile.GetStringFromName("fect.help_"+index+".text"+line.toString());
				addHelpLine(helpTxt);
				line++;
			}catch(err){
				break;
			}
		}
		return true;
	}
	
	function addHelpLine(helpTxt){
		var txt = document.createTextNode(helpTxt);
		var desc = description.cloneNode(true);
		desc.appendChild(txt);	
		helpTxtbox.appendChild(desc);	
		helpTxtbox.appendChild(separator.cloneNode(true));
		FECT.fectClipboard.push(helpTxt);
		FECT.fectClipboard.push("");
		return true;
	}
	FECT.fectPlaySound("help");
	return true;
},


logMsg: function(msg) {
	msg = "FECT: "+msg;
	let cs = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
	cs.logStringMessage(msg);
	return true;
},
	
propsConsole: function(obj,title,ignoreProperties,ignoreFunctions,ignoreObjects,expandObjs,sort){
	if(title != null){
		title = "("+title+")";
	}else{
		title = "";
	}
	if (obj === null) {
		this.logMsg("propsConsole called with null argument: "+title+"\n");
		return false;
	}
	if (obj === undefined) {
		this.logMsg("propsConsole called with undefined argument: "+title+"\n");
		return false;
	}
	let tmp, line, pVal, type, msg, attrb = [], func = [], objects = [];
	msg = "propsConsole: "+title+"\ntypeof '"+obj+"': ("+typeof obj+"), "+((sort) ? "Sorted" : "Unsorted")+"\n";
	for(let property in obj){
		line = property;
		try{
			pVal = obj[property];
		}catch(err){
			pVal = "(Unreachable)  "+err;
		}
		try{
			type = typeof obj[property];
		}catch(e){
			type = "Unknown";
		}
		try{
			tmp = line + " = "+pVal;
		}catch(e){
			tmp = "(Unreachable)";
		}
		switch(type){
			case 'function':
				if(!ignoreFunctions) func.push(tmp);
				break;
			case 'object':
				if(ignoreObjects) continue;
				try{
					let props = JSON.stringify(obj[property]);
					if(expandObjs && props != "null") tmp += " --> "+props;
				}catch(err){;;}
				objects.push(tmp);
				break;
			default:
				if(!ignoreProperties) attrb.push(tmp);
				break;
		}
	} 
	if(func.length == 0) func.push("(none)");
	if(objects.length == 0) objects.push("(none)");
	if(attrb.length == 0) attrb.push("(none)");

	if(sort){
		attrb.sort();
		func.sort();
		objects.sort();
	}

	if(!ignoreProperties){
		msg += "\n*** PROPERTIES/ATTRIBUTES ***\n";
		for(let i in attrb){
			msg += attrb[i]+"\n";
		}
	}
	
	if(!ignoreFunctions){
		msg += "\n*** FUNCTIONS ***\n";
		for(let i in func){
			msg += func[i]+"\n";
		}
	}
	
	if(!ignoreObjects){
		msg += "\n*** OBJECTS ***\n";
		for(let i in objects){
			msg += objects[i]+"\n";
		}
	}
	this.logMsg(msg);
	return true;
},
}